/**
 * Mongoose Aggregate Paginate
 * @param  {Aggregate} aggregate
 * @param  {any} options
 * @param  {function} [callback]
 * @returns {Promise}
 */

const defaultOptions = {
  customLabels: {
    totalDocs: 'totalDocs',
    limit: 'limit',
    page: 'page',
    totalPages: 'totalPages',
    docs: 'docs',
    nextPage: 'nextPage',
    prevPage: 'prevPage',
    pagingCounter: 'pagingCounter',
    hasPrevPage: 'hasPrevPage',
    hasNextPage: 'hasNextPage'
  },
  collation: {},
  lean: false,
  leanWithId: true,
  limit: 10,
  projection: {},
  select: '',
  options: {},
  pagination: true
};

function aggregatePaginate(query, options, callback) {

  console.log(aggregatePaginate);

  options = {
    ...defaultOptions,
    ...aggregatePaginate.options,
    ...options
  };

  query = query || {};

  const customLabels = {
    ...defaultOptions.customLabels,
    ...options.customLabels
  };

  const defaultLimit = 10;

  // Custom Labels
  const labelTotal = customLabels.totalDocs;
  const labelLimit = customLabels.limit;
  const labelPage = customLabels.page;
  const labelTotalPages = customLabels.totalPages;
  const labelDocs = customLabels.docs;
  const labelNextPage = customLabels.nextPage;
  const labelPrevPage = customLabels.prevPage;
  const labelHasNextPage = customLabels.hasNextPage;
  const labelHasPrevPage = customLabels.hasPrevPage;
  const labelPagingCounter = customLabels.pagingCounter;

  let page = parseInt(options.page || 1, 10) || 1;
  let limit = parseInt(options.limit, 10) > 0 ? parseInt(options.limit, 10) : defaultLimit;

  const skip = (page - 1) * limit;
  const sort = options.sort;
  const allowDiskUse = options.allowDiskUse || false;
  const isPaginationEnabled = options.pagination === false ? false : true;

  const q = this.aggregate(query._pipeline);
  const countQuery = this.aggregate(q._pipeline);

  if (q.hasOwnProperty('options')) {
    q.options = query.options;
    countQuery.options = query.options;
  }

  if (sort) {
    q.sort(sort);
  }

  if (allowDiskUse) {
    q.allowDiskUse(true)
    countQuery.allowDiskUse(true)
  }

  if (isPaginationEnabled) {
    q.skip(skip).limit(limit);
  }

  return Promise.all([

      q.exec(),
      countQuery.group({
        _id: null,
        count: {
          $sum: 1
        }
      }).exec()
    ])
    .then(function (values) {

      var count = values[1][0] ? values[1][0].count : 0;

      if (isPaginationEnabled === false) {
        limit = count;
        page = 1;
      }

      var pages = Math.ceil(count / limit) || 1;

      var result = {
        [labelDocs]: values[0],
        [labelTotal]: count,
        [labelLimit]: limit,
        [labelPage]: page,
        [labelTotalPages]: pages,
        [labelPagingCounter]: ((page - 1) * limit) + 1,
        [labelHasPrevPage]: false,
        [labelHasNextPage]: false
      };

      // Set prev page
      if (page > 1) {
        result[labelHasPrevPage] = true;
        result[labelPrevPage] = (page - 1);
      } else {
        result[labelPrevPage] = null;
      }

      // Set next page
      if (page < pages) {
        result[labelHasNextPage] = true;
        result[labelNextPage] = (page + 1);
      } else {
        result[labelNextPage] = null;
      }

      if (typeof callback === 'function') {
        return callback(null, result);
      }

      return Promise.resolve(result);

    })
    .catch(function (reject) {
      if (typeof callback === 'function') {
        return callback(reject)
      }
      return Promise.reject(reject)
    })
}

/**
 * @param {Schema} schema
 */
module.exports = (schema) => {
  schema.statics.aggregatePaginate = aggregatePaginate;
};

module.exports = aggregatePaginate;